prog='dbmultistats'
args='-S -S -k experiment duration'
in=TEST/dbmapreduce_grouped_incorrectly.in
cmp='diff -cb '
portable=false
subprogs=dbstats
altcmp='dbfilediff -E --exit '
altcmp_needs_input_flags=true
